//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  Saving.hpp -- template and derived ESC/I ESC/I-2 protocol commands

#include <utils/ESBuffer.h>
#include <utils/ESFile.h>
#include "ESCommonTypedef.h"
#include "ESImageInfo.h"
#include "EpsonScan2.h"
#include "KeyMgr.hpp"
#include "ModelInfo.hpp"
#include "Filter.hpp"
#include "Error.hpp"
#include "../FileFormat/FFManager.hpp"
namespace epsonscan
{
    class Saving : public Filter
    {
        public:
           Saving(std::shared_ptr<KeyMgr> keyMgr,
                      std::shared_ptr<ModelInfo> modelInfo) : Filter(keyMgr, modelInfo)
            {
            }
            virtual void DoProcess(ESImageInfo& imageInfo, ES_CMN_FUNCS::BUFFER::CESHeapBuffer& inDataBuf) override
            {
                   FFManager ffMgr = FFManager::GetInstance();
                 CESFile* file = nullptr;
     std::stringstream prefixStream;
     prefixStream << ES_IMAGE_INFO::GetESImageWidth(imageInfo)
         << "_"
         << ES_IMAGE_INFO::GetESImageHeight(imageInfo)
         << "_"
         << ES_IMAGE_INFO::GetESImageSamplesPerPixel(imageInfo)
         << "_"
         << ES_IMAGE_INFO::GetESImageBitsPerSample(imageInfo)
         << "_"
         << ES_IMAGE_INFO::GetESImageResolutionX(imageInfo)
         << "_";
     SDI_TRACE_LOG("will create file for page = %d", ES_IMAGE_INFO::GetESImagePageCount(imageInfo));
     SDIImageFormat imgFormat = (SDIImageFormat)keyMgr_->GetValueInt(kSDIImageFormatKey);
     if (imgFormat == kSDIImageFormatRaw)
     {
         file = CESFile::CreateTempFileInstanceWithPrefix(keyMgr_->GetWorkTempPath(), prefixStream.str(), CESFile::ES_OPEN_MODE_WRITE_PLUS);
      if(file->WriteData(inDataBuf) == false)
      {
       delete file;
       file = nullptr;
       SDI_TRACE_LOG("ffMgr.Write fails %d", ES_IMAGE_INFO::GetESImagePageCount(imageInfo));
       throw RuntimeError("file write error", kSDIErrorWriteFile);
      }
      file->CloseFile();
     }else{
      if (imgFormat == kSDIImageFormatColorJpegMonoPNM && ES_IMAGE_INFO::GetESImageColorType(imageInfo) != ES_IMAGE_INFO::kESImageColorTypeMono)
      {
       imgFormat = kSDIImageFormatJPEG;
      }else if (imgFormat == kSDIImageFormatColorJpegMonoPNM && ES_IMAGE_INFO::GetESImageColorType(imageInfo) == ES_IMAGE_INFO::kESImageColorTypeMono)
      {
       imgFormat = kSDIImageFormatPNM;
      }
      file = CESFile::CreateTempFileInstanceWithPrefix(keyMgr_->GetWorkTempPath(), prefixStream.str());
      file->CloseFile();
      if(ffMgr.Write(inDataBuf,
          file->GetFileName(),
          imgFormat,
          imageInfo,
          *keyMgr_
          ) == false)
      {
       delete file;
       file = nullptr;
       SDI_TRACE_LOG("ffMgr.Write fails %d", ES_IMAGE_INFO::GetESImagePageCount(imageInfo));
       throw RuntimeError("file write error", kSDIErrorWriteFile);
      }
     }
     filePath_ = file->GetFileName();
     delete file;
     file = nullptr;
            }
            std::string GetOutputFileName()
            {
                return filePath_;
            }
            std::string Name() override
            {
                return "Saving";
            }
            private:
                std::string filePath_;
    };
}
