//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  Finder.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "Finder.hpp"
#include "NetFinder.hpp"
#include "UsbFinder.hpp"
#include "Utility.hpp"
#include <functional>
#include <memory>
#include <iostream>
namespace epsonscan {
  Finder::Finder() :
  usbFinder_(std::make_shared<UsbFinder>()), netFinder_(std::make_shared<NetFinder>())
  {
  }
  void Finder::Start(bool searchSecondDevice )
  {
   usbFinder_.get()->Start(devices_, searchSecondDevice);
   netFinder_.get()->Reset();
      notifyThread_ = std::make_shared<std::thread>([this]() {
         NetFinderCallBack finderCallBack = [this](SDIDeviceInfo& device) {
      SDI_TRACE_LOG("network device found ip = %s modelID = %s displayname = %s", device.ipAddress, device.modelID, device.displayName);
      devices_.push_back(device);
      };
    netFinder_.get()->Start(finderCallBack);
   });
  }
  void Finder::Stop() {
   if (notifyThread_) {
    netFinder_.get()->Stop();
    notifyThread_.get()->join();
    notifyThread_ = nullptr;
   }
  }
  SDIDeviceInfo* Finder::GetDevicesPointer()
  {
   return &devices_[0];
  }
  int32_t Finder::GetDevicesCount() const
  {
   return static_cast<int32_t>(devices_.size());
  }
}
