//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  PaperEndDetection.hpp -- template and derived ESC/I ESC/I-2 protocol commands

#include "Key.hpp"
#include "EpsonScan2.h"
namespace epsonscan {
 class PaperEndDetection : public Key<SDIInt>
 {
  public:
  explicit PaperEndDetection(IKeyDataProvider* dataProvider) : Key<SDIInt>(dataProvider)
  {
  }
  std::string GetName() const override
  {;
   return std::string(kSDIPaperEndDetectionKey);
  }
  void SetValue(const SDIInt& intVal) override
        {
         current_ = intVal;
        }
  void GetValue(SDIInt& outIntVal) override
        {
         outIntVal = current_;
        }
  void GetCapability(SDICapability &capability) override
        {
                SetDefaultListCapability(capability);
                capability.supportLevel = kSDISupportLevelUnavailable;
                ESIndexSet available;
                Scanner* scanner = dataProvider_->GetScanner().get();
                if(scanner && scanner->GetAvailableValueForKey(kESPaperEndDetection, available)) {
                    if (available.find(true) != available.end())
                    {
                        AddList(capability, true);
                        AddList(capability, false);
                        AddAllList(capability, true);
                        AddAllList(capability, false);
                        capability.supportLevel = kSDISupportLevelAvailable;
                    }else{
                        AddList(capability, false);
                        AddAllList(capability, false);
                    }
                }
                if (GetValueInt(kSDIFunctionalUnitKey) == kSDIFunctionalUnitFlatbed)
                {
                    capability.supportLevel = kSDISupportLevelUnavailable;
                    ResetList(capability);
                }
            }
  void Reset() override
        {
         current_ = false;
        }
  ESDictionary GetEngineKeyValues() override
        {
            auto dict = ESDictionary();
            return dict;
        }
 };
}
