//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.
//  
//  mainwindow.cpp -- template and derived ESC/I ESC/I-2 protocol commands

#include <QDesktopServices>
#include <QDebug>
#include <QProcess>
#include "mainwindow.h"
#include "deviceselectdialog.h"
#include "scanningdialog.h"
#include "aboutepsonscan2dialog.h"
#include "addpagedialog.h"
#include "devicelist.h"
#include "devicefinder.h"
#include "PathUtils.h"
MainWindow::MainWindow(const std::string edit_output_path, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    configuration_dialog = NULL;
    m_last_used_settings_controller = nullptr;
    m_messageBox = NULL;
    m_messageBox = new CustomMessageBox(this);
    if(m_messageBox == NULL){
        exit(EXIT_SUCCESS);
    }
    sv = nullptr;
    sv = new Supervisor();
    if(sv == NULL){
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SCAN_001, TranslationString::GetString().translation_str.VERSION_003, Critical);
        exit(EXIT_SUCCESS);
    }
    sv->scanning_status = SAVE;
    sv->bAllReset = false;
    Configuration& config = Configuration::GetConfiguration();
    if(edit_output_path.c_str()[0] != '\0'){
        sv->edit_mode = true;
        sv->setting_file_path = edit_output_path;
    }else {
        sv->edit_mode = false;
    }
    m_last_used_settings_controller = new LastUsedSettings();
    if(m_last_used_settings_controller == NULL){
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SCAN_001, TranslationString::GetString().translation_str.VERSION_003, Critical);
        this->CloseWindow();
        exit(EXIT_SUCCESS);
    }
    ui->setupUi(this);
    SetupMainWindow(config);
    sv->SetUp();
    if(sv->edit_mode){
        ui->Scan_Button_stackedWidget->setCurrentIndex(1);
    }else {
        ui->Scan_Button_stackedWidget->setCurrentIndex(0);
    }
    if(!configuration_dialog){
        configuration_dialog = new ConfigurationDialog(this);
        if(configuration_dialog == NULL){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SCAN_001, TranslationString::GetString().translation_str.VERSION_003, Critical);
            this->CloseWindow();
            exit(EXIT_SUCCESS);
        }
    }
    bool connection_success = false;
    while(!connection_success){
        sv->Disconnecting_Scanner();
        sv->Dispose_Scanner();
        ui->Device_selector_comb->initialize();
        if(m_last_used_settings_controller->CheckLastUsedDevice()){
            if(!m_last_used_settings_controller->ReadPreferredInfo()){
                if(!this->SelectDevice()){
                    break;
                }
            }
            config.SetConfiguration(m_last_used_settings_controller->ReadCommonSettingsFile());
        }else {
            if(!this->SelectDevice()){
                sv->bAllReset = true;
                break;
            }
        }
        ui->Device_selector_comb->add_device();
        ui->Device_selector_comb->setCurrentIndex(DeviceList::select_device);
        connection_success = this->ConnectToDevice();
        if(connection_success){
            this->updateUI();
            ui->Folder_comb->select_current_path(&sv->device_data);
        }else {
            m_last_used_settings_controller->DeletePreferredInfo();
        }
    }
    if(connection_success){
        this->ValidateOverRollerLifeLimit();
    }
}
MainWindow::~MainWindow()
{
    this->CloseWindow();
    exit(EXIT_SUCCESS);
}
void MainWindow::SetupMainWindow(Configuration config)
{
    int x, y, width, height;
    width = INIT_MAINWINDOW_WIDTH;
    config.config_para = m_last_used_settings_controller->ReadMainWindowInfo();
    if(config.config_para.WindowPositionX == INIT_POSITION_X && config.config_para.WindowPositionY == INIT_POSITION_Y){
        x = INIT_POSITION_X;
        y = INIT_POSITION_Y;
    }else {
        x = config.config_para.WindowPositionX;
        y = config.config_para.WindowPositionY;
    }
    if(config.config_para.WindowHeight != INIT_MAINWINDOW_HEIGHT){
        height = config.config_para.WindowHeight;
    }else {
        height = INIT_MAINWINDOW_HEIGHT;
    }
    this->setGeometry(x, y, width, height);
    this->setMaximumWidth(INIT_MAINWINDOW_WIDTH);
    this->setMinimumHeight(INIT_MAINWINDOW_HEIGHT);
    ui->Folder_toolButton->setIcon(this->style()->standardIcon(QStyle::SP_DirLinkIcon));
#ifdef NOUSEQRC
    std::string strHelpIconPath = EPSON_INSTALL_PATH;
    ES_CMN_FUNCS::PATH::ES_CombinePath(strHelpIconPath, strHelpIconPath, "Resources/Icons/btn_help.png");
    ui->Help_toolButton->setIcon(QIcon(strHelpIconPath.c_str()));
    std::string strConfIconPath = EPSON_INSTALL_PATH;
    ES_CMN_FUNCS::PATH::ES_CombinePath(strConfIconPath, strConfIconPath, "Resources/Icons/btn_config@2x.png");
    ui->Configuration_toolButton->setIcon(QIcon(strConfIconPath.c_str()));
#endif
#ifdef RPMBUILD
    ui->Help_toolButton->setIcon(this->style()->standardIcon(QStyle::SP_MessageBoxQuestion));
    ui->Configuration_toolButton->setIcon(this->style()->standardIcon(QStyle::SP_ComputerIcon));
#endif
}
void MainWindow::CloseWindow()
{
    if(sv != NULL && sv->driver){
        m_last_used_settings_controller->CreatePreferredInfo(ui->Device_selector_comb->get_cur_device());
        if(sv->bAllReset == false){
            Configuration& config = Configuration::GetConfiguration();
            QRect MainWindowInfo = this->geometry();
            QRect MainWindowInfo_withFrame = this->frameGeometry();
            config.config_para.WindowPositionX = MainWindowInfo_withFrame.x();
            config.config_para.WindowPositionY = MainWindowInfo_withFrame.y();
            config.config_para.WindowWidth = MainWindowInfo.width();
            config.config_para.WindowHeight = MainWindowInfo.height();
            config.SetConfiguration(config.config_para);
            m_last_used_settings_controller->CreateCommonSettingsFile(config);
            this->isManualResolution();
            m_last_used_settings_controller->CreateSettingsFile(ui->Device_selector_comb->get_cur_device().modelID,
                                                                sv->device_data);
        }
    }
    if(m_last_used_settings_controller != nullptr){
        delete m_last_used_settings_controller;
        m_last_used_settings_controller = nullptr;
    }
    if(configuration_dialog){
        delete configuration_dialog;
        configuration_dialog = NULL;
    }
    if(sv != NULL){
        sv->Terminate();
        sv->Disconnecting_Scanner();
        sv->Dispose_Scanner();
        delete sv;
        sv = nullptr;
    }
    if(m_messageBox != NULL){
        delete m_messageBox;
        m_messageBox = NULL;
    }
    ES_CMN_FUNCS::PATH::ES_DeleteFolder(EPSON_WORK_PATH);
    delete ui;
}
bool MainWindow::SelectDevice(void)
{
    DeviceSelectDialog* device_select_dialog = NULL;
    device_select_dialog = new DeviceSelectDialog;
    if(device_select_dialog == NULL){
        return false;
    }
    device_select_dialog->setWindowFlags(Qt:: Dialog);
    if(!device_select_dialog->Initialize(sv)){
        return false;
    }
    if(device_select_dialog->isHidden()){
        device_select_dialog->exec();
    }else {
        device_select_dialog->activateWindow();
    }
    if(device_select_dialog->terminate_flag){
        delete device_select_dialog;
        return false;
    }
    return true;
}
void MainWindow::showEvent( QShowEvent* event ) {
    if(!sv->driver){
        this->CloseWindow();
        exit(EXIT_SUCCESS);
    }
}
void MainWindow::Check2in1Difference()
{
    if(!sv->device_data.Use2in1Mode.select && sv->device_data.FunctionalUnit.select == Flatbed){
        sv->device_data.FunctionalUnit.select = sv->device_data.FunctionalUnit.capability.list[0];
        sv->Set_Exclusion_Value();
    }else if(!sv->device_data.Use2in1Mode.select && sv->device_data.FunctionalUnit_Auto){
        sv->device_data.FunctionalUnit.select = ADF;
        sv->Set_Exclusion_Value();
    }
}
bool MainWindow::search_item_list(int item, int32_t item_list[], int32_t list_size)
{
    int i = 0;
    while(i < list_size){
        if(item_list[i] == item){
            return true;
        }
        i++;
    }
    return false;
}
void MainWindow::Initialize(void)
{
    if(this->search_item_list(kSDIFunctionalUnitFlatbed, sv->device_data.FunctionalUnit.capability.list, sv->device_data.FunctionalUnit.capability.countOfList)){
        sv->device_data.ScanArea.FBWidthMin = 100;
        sv->device_data.ScanArea.FBHeightMin = 100;
    }else {
        sv->device_data.ScanArea.FBWidthMin = 0;
        sv->device_data.ScanArea.FBHeightMin = 0;
    }
    ui->Document_source_comb->initialize(sv->device_data.FunctionalUnit, sv->device_data.FunctionalUnit_Auto);
    ui->Scanning_side_comb->initialize();
    ui->Document_size_comb->initialize(sv->device_data.ScanArea, sv->device_data.Resolution.select);
    ui->Image_type_comb->initialize();
    ui->Resolution_comb->initialize(sv->device_data.Resolution);
    ui->Rotate_comb->initialize();
    ui->Deskew_checkbox->initialize();
    ui->Ad_pages_checkbox->initialize(sv->device_data.Add_Pages);
    ui->Skip_blank_pages_comb->initialize(sv->device_data);
    ui->Detect_double_feed_comb->initialize(sv->device_data.DoubleFeedDetectionValue.DoubleFeedDetectionLevel);
    ui->RemoveBack_comboBox->initialize(sv->device_data.BackgroundRemoval);
    ui->Dropout_comboBox->initialize(sv->device_data.DropoutColor);
    ui->Brightness_hSlider->initialize(sv->device_data.Brightness.select);
    ui->Contrast_hSlider->initialize(sv->device_data.Contrast.select);
    ui->Gamma_hSlider->initialize(sv->device_data.Gamma.select);
    ui->Threshold_hSlider->initialize(sv->device_data.Threshold.select);
    ui->Image_format_comb->initialize(sv->device_data.ImageFormatSettings);
    ui->Prefix_lineEdit->initialize(sv->device_data.Prefix);
    ui->Folder_comb->initialize(sv->device_data, m_messageBox);
    ui->Scan_Button->initialize(m_messageBox);
    ui->Automatic_feeding_mode_comb->initialize(sv->device_data.AutomaticFeedingMode);
    ui->Background_color_comb->initialize(sv->device_data);
    this->InitializeImageOption(sv->device_data.ImageOption);
    ui->Configuration_toolButton->setEnabled(true);
    ui->tabWidget->setEnabled(true);
    this->SetAllLabelEnabled(true);
}
void MainWindow::SetAllLabelEnabled(bool enabled)
{
    ui->device_name_colon_label->setEnabled(enabled);
    ui->Document_source_label->setEnabled(enabled);
    ui->Doc_source_colon_label->setEnabled(enabled);
    ui->Scanning_side_label->setEnabled(enabled);
    ui->Scanning_side_colon_label->setEnabled(enabled);
    ui->Document_size_label->setEnabled(enabled);
    ui->Document_size_colon_label->setEnabled(enabled);
    ui->Image_type_label->setEnabled(enabled);
    ui->Image_type_colon_label->setEnabled(enabled);
    ui->Resolution_label->setEnabled(enabled);
    ui->Resolution_colon_label->setEnabled(enabled);
    ui->Dpi_label->setEnabled(enabled);
    ui->Rotate_label->setEnabled(enabled);
    ui->Rotate_colon_label->setEnabled(enabled);
    ui->Skip_blank_pages_label->setEnabled(enabled);
    ui->Skip_blank_pages_colon_label->setEnabled(enabled);
    ui->Detect_double_page_label->setEnabled(enabled);
    ui->Detect_double_page_colon_label->setEnabled(enabled);
    ui->Image_option_colon_label->setEnabled(enabled);
    ui->Image_format_label->setEnabled(enabled);
    ui->Image_format_colon_label->setEnabled(enabled);
    ui->File_name_label->setEnabled(enabled);
    ui->File_name_colon_label->setEnabled(enabled);
    ui->File_extension_label->setEnabled(enabled);
    ui->Folder_label->setEnabled(enabled);
    ui->Folder_colon_label->setEnabled(enabled);
    ui->Folder_toolButton->setEnabled(enabled);
    ui->Save_pushButton->setEnabled(enabled);
    ui->Help_toolButton->setEnabled(enabled);
}
bool MainWindow::ConnectToDevice(void)
{
    Configuration& config = Configuration::GetConfiguration();
    bool use_lastsettings = false;
    SDIDeviceInfo select_Device = ui->Device_selector_comb->get_cur_device();
    if(select_Device.ipAddress[0] != '\0'){
        if(!sv->DeviceInfoResolve(&select_Device)){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.LINUX_E_COMM_001, TranslationString::GetString().translation_str.VERSION_003, Warning);
            return false;
        }
    }else if(select_Device.displayName[0] == '\0'){
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.LINUX_E_COMM_001, TranslationString::GetString().translation_str.VERSION_003, Warning);
        return false;
    }
    if(config.config_para.last_used_settings){
        use_lastsettings = m_last_used_settings_controller->ReadSettingsFile(select_Device.modelID, &sv->device_data);
    }
    DeviceConnectionDialog* device_connection_dialog = new DeviceConnectionDialog(sv, select_Device, use_lastsettings);
    if(device_connection_dialog == NULL){
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SCAN_001, TranslationString::GetString().translation_str.VERSION_003, Critical);
        return false;
    }
    if(device_connection_dialog->GetConnectionResult()){
        if(use_lastsettings){
            sv->Set_All_Value();
        }else {
            sv->device_data.AutomaticFeedingMode.select = false;
        }
        sv->Get_All_Value();
        this->Check2in1Difference();
        sv->Get_All_Value();
        this->Initialize();
        return true;
    }else {
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.LINUX_E_COMM_001, TranslationString::GetString().translation_str.VERSION_003, Warning);
        return false;
    }
}
void MainWindow::onValueChanged(int value)
{
    QObject* obj = sender();
    if(obj == ui->Device_selector_comb){
        ui->Device_selector_comb->item_event(value);
        sv->Disconnecting_Scanner();
        sv->Dispose_Scanner();
        if(!this->ConnectToDevice()){
            this->close();
        }
        ui->Folder_comb->select_current_path(&sv->device_data);
        this->ValidateOverRollerLifeLimit();
    }else if(obj == ui->Document_source_comb){
        ui->Document_source_comb->item_event(value, &sv->device_data);
        ui->Document_size_comb->changed = true;
    }else if(obj == ui->Scanning_side_comb){
        ui->Scanning_side_comb->item_event(value, &sv->device_data.DuplexType);
        ui->Document_size_comb->changed = true;
    }else if(obj == ui->Document_size_comb){
        DocumentSize old_select = ui->Document_size_comb->current_item;
        if(this->SelectionRestrictions(value)){
            ui->Document_size_comb->item_event(value, &sv->device_data.ScanArea);
            if(ui->Document_size_comb->current_item == PlasticCard && !sv->device_data.DNShow_PlasticCard_dialog){
                sv->device_data.DNShow_PlasticCard_dialog = m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UI_003, TranslationString::GetString().translation_str.VERSION_003, Information, true);
            }if(!this->SelectionRestrictions(ui->Document_size_comb->findData(ui->Document_size_comb->current_item))){
                ui->Document_size_comb->item_event(ui->Document_size_comb->findData(old_select), &sv->device_data.ScanArea);
            }
        }else {
            ui->Document_size_comb->setCurrentIndex(ui->Document_size_comb->findData(ui->Document_size_comb->current_item));
        }
    }else if(obj == ui->Image_type_comb){
        ui->Image_type_comb->item_event(value, &sv->device_data.ColorType);
        this->ChangeWidget(sv->device_data.ColorType.select);
    }else if(obj == ui->Resolution_comb){
        if(this->SelectionRestrictions(value)){
            ui->Resolution_comb->item_event(value, &sv->device_data.Resolution);
            ui->Document_size_comb->changed = true;
            ui->Document_size_comb->update_ui(sv->device_data, sv->device_data.Resolution.select);
            ui->Document_size_comb->item_event(ui->Document_size_comb->currentIndex(), &sv->device_data.ScanArea);
        }else {
            ui->Resolution_comb->setCurrentText(QString(QString::number(sv->device_data.Resolution.select)));
        }
    }else if(obj == ui->Rotate_comb){
        ui->Rotate_comb->item_event(value, &sv->device_data.Orientation);
    }else if(obj == ui->Skip_blank_pages_comb){
        ui->Skip_blank_pages_comb->item_event(value, &sv->device_data);
    }else if(obj == ui->Detect_double_feed_comb){
        ui->Detect_double_feed_comb->item_event(value, &sv->device_data.DoubleFeedDetectionValue);
    }
    else if(obj == ui->RemoveBack_comboBox){
        ui->RemoveBack_comboBox->item_event(value, &sv->device_data.BackgroundRemoval);
    }else if(obj == ui->Dropout_comboBox){
        ui->Dropout_comboBox->item_event(value, &sv->device_data.DropoutColor);
    }else if(obj == ui->Brightness_hSlider){
        ui->Brightness_hSlider->item_event(value, &sv->device_data.Brightness);
        return;
    }else if(obj == ui->Contrast_hSlider){
        ui->Contrast_hSlider->item_event(value, &sv->device_data.Contrast);
        return;
    }else if(obj == ui->Gamma_hSlider){
        ui->Gamma_hSlider->item_event(value, &sv->device_data.Gamma);
        return;
    }else if(obj == ui->Threshold_hSlider){
        ui->Threshold_hSlider->item_event(value, &sv->device_data.Threshold);
        return;
    }
    else if(obj == ui->Image_format_comb){
        ui->Image_format_comb->item_event(value, &sv->device_data.ImageFormatSettings);
    }else if(obj == ui->Folder_comb){
        ui->Folder_comb->item_event(value, &sv->device_data);
        sv->device_data.Folder = ui->Folder_comb->current_item;
    }
    else if(obj == ui->Automatic_feeding_mode_comb){
        ui->Automatic_feeding_mode_comb->item_event(value, &sv->device_data.AutomaticFeedingMode);
    }else if(obj == ui->Background_color_comb){
        ui->Background_color_comb->item_event(value, &sv->device_data.BackgroundColor);
    }
    this->updateUI();
}
void MainWindow::onValueChangedQstr(QString value)
{
    QObject* obj = sender();
}
void MainWindow::onValueChangedBool(bool check)
{
    QObject* obj = sender();
    if(obj == ui->Deskew_checkbox){
       ui->Deskew_checkbox->item_event(check, &sv->device_data.PaperDeskew);
    }else if(obj == ui->Ad_pages_checkbox){
        ui->Ad_pages_checkbox->item_event(check, &sv->device_data.Add_Pages);
    }else if(obj == ui->None_radioButton){
        if(check){
            ui->RemoveBack_comboBox->setEnabled(false);
            sv->device_data.BackgroundRemoval.select = BackgroundRemovalNone;
            ui->Dropout_comboBox->setEnabled(false);
            sv->device_data.DropoutColor.select = DropoutColorNone;
        }
    }else if(obj == ui->RemoveBack_radioButton){
        if(check){
            sv->device_data.DropoutColor.select = DropoutColorNone;
            if(sv->device_data.ColorType.select == ColorTypeRGB24){
                ui->RemoveBack_comboBox->set_enabled(check);
            }else if(sv->device_data.ColorType.select == ColorTypeMono8){
                ui->RemoveBack_comboBox->set_enabled(false);
                ui->RemoveBack_comboBox->current_item = BackgroundRemovalNormal;
            }
        }else {
            ui->RemoveBack_comboBox->set_enabled(check);
        }
    }else if(obj == ui->Dropout_radioButton){
        if(check){
            sv->device_data.BackgroundRemoval.select = BackgroundRemovalNone;
        }else {
            ui->Dropout_comboBox->set_enabled(check);
        }
    }
    this->updateUI();
}
void MainWindow::EditFinishText(void){
    QObject* obj = sender();
    if(obj == ui->Prefix_lineEdit){
        sv->device_data.Prefix = qPrintable(ui->Prefix_lineEdit->set_prefix(ui->Prefix_lineEdit->text()));
    }
    this->updateUI();
}
bool MainWindow::SelectionRestrictions(int value){
    QObject* obj = sender();
    if(obj == ui->Resolution_comb){
        if(ui->Document_size_comb->current_item == USER_DEFINE){
            return UserDefineSizeRestrictions();
        }else if(ui->Document_size_comb->current_item == Auto_Detect_long){
            if(ui->Resolution_comb->currentText().toInt() > 600){
                m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_011, TranslationString::GetString().translation_str.VERSION_003, Warning);
                return false;
            }
        }
    }else if(obj == ui->Document_size_comb){
        if(ui->Document_size_comb->itemData(value) == USER_DEFINE){
            return UserDefineSizeRestrictions();
        }else if(ui->Document_size_comb->itemData(value) == Auto_Detect_long){
            if(ui->Resolution_comb->currentText().toInt() > 600){
                m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_011, TranslationString::GetString().translation_str.VERSION_003, Warning);
                return false;
            }
        }
    }else if(obj == ui->Scan_Button){
        if(ui->Document_size_comb->itemData(value) == USER_DEFINE){
            return UserDefineSizeRestrictions();
        }
    }
    return true;
}
bool MainWindow::UserDefineSizeRestrictions(void)
{
    Configuration& config = Configuration::GetConfiguration();
    Units units = config.config_para.units;
    double long_paper_limit_value_53 = 0;
    double long_paper_limit_value_215 = 0;
    double long_paper_limit_value_240 = 0;
    double current_value = 0;
    double limit_value = 0;
    if(units == unit_inchi){
        long_paper_limit_value_53 = ui->Document_size_comb->calculate_pixel_size_inchi(53);
        long_paper_limit_value_215 = ui->Document_size_comb->calculate_pixel_size_inchi(215);
        long_paper_limit_value_240 = ui->Document_size_comb->calculate_pixel_size_inchi(240);
        current_value = ui->Document_size_comb->calculate_pixel_size_inchi(sv->device_data.ScanArea.UserDefine_Length);
    }else if(units == unit_mm){
        long_paper_limit_value_53 = ui->Document_size_comb->calculate_pixel_size(1346.2);
        long_paper_limit_value_215 = ui->Document_size_comb->calculate_pixel_size(5461);
        long_paper_limit_value_240 = ui->Document_size_comb->calculate_pixel_size(6096);
        current_value = ui->Document_size_comb->calculate_pixel_size(sv->device_data.ScanArea.UserDefine_Length);
    }
    limit_value = ui->Document_size_comb->calculate_pixel_size_inchi(sv->device_data.ScanArea.MaxScanAreaHeight.select / 100);
    if(sv->device_data.ScanArea.PaperEndDetection.select){
        if(std::atoi(qPrintable(ui->Resolution_comb->currentText())) > 600){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_011, TranslationString::GetString().translation_str.VERSION_003, Warning);
            return false;
        }
    }else if(limit_value < current_value
            && current_value <= long_paper_limit_value_53){
        if(std::atoi(qPrintable(ui->Resolution_comb->currentText())) > 600){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_011, TranslationString::GetString().translation_str.VERSION_003, Warning);
            return false;
        }
    }else if(long_paper_limit_value_53 < current_value
            && current_value <= long_paper_limit_value_215){
        if(std::atoi(qPrintable(ui->Resolution_comb->currentText())) > 300){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_011, TranslationString::GetString().translation_str.VERSION_003, Warning);
            return false;
        }
    }else if(long_paper_limit_value_215 < current_value
             && current_value <= long_paper_limit_value_240){
        if(std::atoi(qPrintable(ui->Resolution_comb->currentText())) > 200){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_011, TranslationString::GetString().translation_str.VERSION_003, Warning);
            return false;
        }
    }
    return true;
}
bool MainWindow::CheckLongPaperMode(void)
{
    if(!sv->device_data.ScanArea.PaperEndDetection.select){
        Configuration& config = Configuration::GetConfiguration();
        Units units = config.config_para.units;
        double long_paper_limit_value = 0;
        double current_value = 0;
        double limit_value = 0;
        if(units == unit_inchi){
            long_paper_limit_value = ui->Document_size_comb->calculate_pixel_size_inchi(215);
            current_value = ui->Document_size_comb->calculate_pixel_size_inchi(sv->device_data.ScanArea.UserDefine_Length);
        }else if(units == unit_mm){
            long_paper_limit_value = ui->Document_size_comb->calculate_pixel_size(5461);
            current_value = ui->Document_size_comb->calculate_pixel_size(sv->device_data.ScanArea.UserDefine_Length);
        }
        limit_value = ui->Document_size_comb->calculate_pixel_size_inchi(sv->device_data.ScanArea.MaxScanAreaHeight.select / 100);
        if(limit_value < current_value && current_value <= long_paper_limit_value){
            return true;
        }else if(current_value > long_paper_limit_value){
            return true;
        }
    }
    return false;
}
void MainWindow::isManualResolution()
{
    if(ui->Resolution_comb->currentText().toInt() != 0){
        sv->device_data.Resolution.select = ui->Resolution_comb->currentText().toInt();
        ui->Document_size_comb->base_resolution = sv->device_data.Resolution.select;
        ui->Document_size_comb->update_scanarea(&sv->device_data.ScanArea);
    }
}
void MainWindow::updateUI(){
    this->isManualResolution();
    sv->Set_Exclusion_Value();
    sv->Get_Exclusion_Value();
    ui->Document_source_comb->update_ui(sv->device_data.FunctionalUnit);
    ui->Scanning_side_comb->update_ui(&sv->device_data.DuplexType);
    ui->Document_size_comb->update_ui(sv->device_data, sv->device_data.Resolution.select);
    ui->Image_type_comb->update_ui(sv->device_data.ColorType);
    ui->Resolution_comb->update_ui(sv->device_data.Resolution);
    ui->Rotate_comb->update_ui(sv->device_data.Orientation);
    ui->Deskew_checkbox->update_ui(sv->device_data.PaperDeskew);
    ui->Ad_pages_checkbox->update_ui();
    ui->Skip_blank_pages_comb->update_ui(sv->device_data.BlankPageSkipLevelKey);
    ui->Detect_double_feed_comb->update_ui(sv->device_data.DoubleFeedDetectionValue.DoubleFeedDetection);
    ui->RemoveBack_comboBox->update_ui(sv->device_data.BackgroundRemoval);
    ui->Dropout_comboBox->update_ui(sv->device_data.DropoutColor);
    ui->Brightness_hSlider->update_ui();
    ui->Contrast_hSlider->update_ui();
    ui->Gamma_hSlider->update_ui();
    ui->Threshold_hSlider->update_ui();
    ui->Image_format_comb->update_ui(sv->device_data.ImageFormatSettings);
    ui->Folder_comb->update_ui();
    ui->Automatic_feeding_mode_comb->update_ui(sv->device_data.AutomaticFeedingMode);
    ui->Background_color_comb->update_ui(sv->device_data.BackgroundColor);
    ui->Device_selector_comb->update_device_list();
    this->ControlImageOption();
    this->ChangeWidget(sv->device_data.ColorType.select);
    this->ControlDeskewAvailable();
    this->ControlDocumentSizeAvailable();
    this->ControlDropoutAvailable(sv->device_data.ColorType.select);
    this->ControlRemoveBackGroundAvailable(sv->device_data.ColorType.select);
    this->ChangeExtension(ui->Image_format_comb->current_item);
    this->ControlDetDoubleFeedAvailable(ui->Document_size_comb->current_item);
    this->ControlSkipBlankPagesAvailable(ui->Document_source_comb->current_item);
    this->ControlScanningSideAvailable();
    this->ControlBackgroundColorAvailable();
    this->ControlAutomaticFeedingModeAvailable();
}
#include <QStandardItemModel>
void MainWindow::ControlScanningSideAvailable(void)
{
    if(ui->Scanning_side_comb->isEnabled()){
        ui->Scanning_side_label->setEnabled(true);
        ui->Scanning_side_colon_label->setEnabled(true);
    }else {
        ui->Scanning_side_label->setEnabled(false);
        ui->Scanning_side_colon_label->setEnabled(false);
    }
    if(ui->Scanning_side_comb->isHidden()){
        ui->Scanning_side_label->hide();
        ui->Scanning_side_colon_label->hide();
    }else {
        ui->Scanning_side_label->show();
        ui->Scanning_side_colon_label->show();
    }
}
void MainWindow::ControlDocumentSizeAvailable(void)
{
    if(ui->Document_source_comb->current_item == Auto){
        QStandardItemModel* model = dynamic_cast< QStandardItemModel * >( ui->Document_size_comb->model() );
        QStandardItem* item = model->item(ui->Document_size_comb->findData(Auto_Detect_long));
        if(item != 0x0){
            item->setEnabled( false );
            if(ui->Document_size_comb->current_item == Auto_Detect_long){
                ui->Document_size_comb->current_item = Auto_Detect;
                ui->Document_size_comb->setCurrentIndex(ui->Document_size_comb->findData(Auto_Detect));
            }
        }
    }
}
void MainWindow::ChangeExtension(ImageFormat imgformat){
    if(imgformat == ImageFormatPNM){
        ui->File_extension_label->setText(TranslationString::GetString().translation_str.LINUX_FILESAVESETTING_001);
    }else if(imgformat == ImageFormatJPEG){
        ui->File_extension_label->setText(TranslationString::GetString().translation_str.LINUX_FILESAVESETTING_002);
    }else if(imgformat == ImageFormatPNG){
        ui->File_extension_label->setText(TranslationString::GetString().translation_str.LINUX_FILESAVESETTING_003);
    }else if(imgformat == ImageFormatTIFF || imgformat == ImageFormatMultiTIFF){
        ui->File_extension_label->setText(TranslationString::GetString().translation_str.LINUX_FILESAVESETTING_004);
    }else if(imgformat == ImageFormatColorJpegMonoRaw){
        ui->File_extension_label->setText(TranslationString::GetString().translation_str.LINUX_FILESAVESETTING_005);
    }
}
void MainWindow::ChangeWidget(SDIInt type){
    if(type == ColorTypeRGB24 || type == ColorTypeMono8){
        ui->image_type_stackedWidget->setCurrentIndex(1);
    }else if(type == ColorTypeMono1){
        ui->image_type_stackedWidget->setCurrentIndex(0);
    }
}
void MainWindow::CreateComboBoxToolTip(QComboBox* obj, QString message)
{
    QString qStr;
    if(obj->isEnabled()){
        qStr = message;
    }else {
        qStr = "";
    }
    obj->setToolTip(qStr);
}
void MainWindow::InitializeImageOption(SDIInt select)
{
    if(select == ImageOptionNone){
        ui->None_radioButton->setChecked(true);
    }else if(select == ImageOptionDropout){
        ui->Dropout_radioButton->setChecked(true);
    }else if(select == ImageOptionBackgroundRemoval){
        ui->RemoveBack_radioButton->setChecked(true);
    }
}
void MainWindow::ControlImageOption()
{
    if(ui->None_radioButton->isChecked()){
        sv->device_data.ImageOption = ImageOptionNone;
    }else if(ui->Dropout_radioButton->isChecked()){
        sv->device_data.ImageOption = ImageOptionDropout;
    }else if(ui->RemoveBack_radioButton->isChecked()){
        sv->device_data.ImageOption = ImageOptionBackgroundRemoval;
    }
}
void MainWindow::ControlDropoutAvailable(SDIInt type)
{
    CreateComboBoxToolTip(ui->Dropout_comboBox, TranslationString::GetString().translation_str.ADVANCEDSETTING_072);
    if(type == ColorTypeRGB24){
        ui->Dropout_radioButton->setEnabled(false);
        if(ui->Dropout_radioButton->isChecked()){
            ui->None_radioButton->setChecked(true);
            ui->Dropout_comboBox->setCurrentIndex(ui->Dropout_comboBox->findData(DropoutColorRed));
            ui->Dropout_comboBox->current_item = DropoutColorRed;
        }
    }else if(type == ColorTypeMono1 || type == ColorTypeMono8){
        ui->Dropout_radioButton->setEnabled(true);
    }
    if(ui->Dropout_comboBox->isHidden()){
        ui->Dropout_radioButton->hide();
    }else {
        ui->Dropout_radioButton->show();
    }
}
void MainWindow::ControlRemoveBackGroundAvailable(SDIInt type)
{
    CreateComboBoxToolTip(ui->RemoveBack_comboBox, TranslationString::GetString().translation_str.ADVANCEDSETTING_070);
    if(type == ColorTypeMono1){
        ui->RemoveBack_radioButton->setEnabled(false);
        if(ui->RemoveBack_radioButton->isChecked()){
            ui->None_radioButton->setChecked(true);
            ui->RemoveBack_comboBox->setCurrentIndex(ui->RemoveBack_comboBox->findData(BackgroundRemovalNormal));
            ui->RemoveBack_comboBox->current_item = BackgroundRemovalNormal;
        }
    }else if(type == ColorTypeMono8){
        ui->RemoveBack_radioButton->setEnabled(true);
        ui->RemoveBack_comboBox->setCurrentIndex(ui->RemoveBack_comboBox->findData(BackgroundRemovalNormal));
        ui->RemoveBack_comboBox->current_item = BackgroundRemovalNormal;
        if(ui->RemoveBack_radioButton->isChecked()){
             ui->RemoveBack_comboBox->setEnabled(false);
        }
    }else if(type == ColorTypeRGB24){
        ui->RemoveBack_radioButton->setEnabled(true);
        if(ui->RemoveBack_radioButton->isChecked()){
            ui->RemoveBack_comboBox->setEnabled(true);
        }
    }
    if(ui->RemoveBack_comboBox->isHidden()){
        ui->RemoveBack_radioButton->hide();
    }else {
        ui->Skip_blank_pages_label->show();
        ui->RemoveBack_radioButton->show();
    }
}
void MainWindow::ControlDetDoubleFeedAvailable(SDIInt type)
{
    if(type == PlasticCard || type == PlasticCard_Landscape){
        ui->Detect_double_feed_comb->item_event(0, &sv->device_data.DoubleFeedDetectionValue);
        ui->Detect_double_feed_comb->setCurrentIndex(Off);
        ui->Detect_double_feed_comb->setEnabled(false);
    }else if(type == Auto_Detect_long || (type == USER_DEFINE && this->CheckLongPaperMode())){
        ui->Detect_double_feed_comb->item_event(0, &sv->device_data.DoubleFeedDetectionValue);
        ui->Detect_double_feed_comb->setCurrentIndex(Off);
        ui->Detect_double_feed_comb->setEnabled(false);
    }else if(sv->device_data.ScanArea.PaperEndDetection.select){
        ui->Detect_double_feed_comb->item_event(0, &sv->device_data.DoubleFeedDetectionValue);
        ui->Detect_double_feed_comb->setCurrentIndex(Off);
        ui->Detect_double_feed_comb->setEnabled(false);
    }else {
        if(ui->Document_source_comb->current_item != Flatbed){
            ui->Detect_double_feed_comb->setEnabled(true);
        }
    }
    if(ui->Detect_double_feed_comb->isEnabled()){
        ui->Detect_double_page_label->setEnabled(true);
        ui->Detect_double_page_colon_label->setEnabled(true);
    }else {
        ui->Detect_double_page_label->setEnabled(false);
        ui->Detect_double_page_colon_label->setEnabled(false);
    }
    if(ui->Detect_double_feed_comb->isHidden()){
        ui->Detect_double_page_label->hide();
    }else {
        ui->Detect_double_page_label->show();
    }
}
void MainWindow::ControlSetDocumentAvailable(SDIInt type)
{
    if(type == Auto_Detect || type == Maximum_Size){
        ui->Horizontal_doc_toolButton->setEnabled(false);
        ui->Vertical_doc_toolButton->setEnabled(false);
        ui->Horizontal_doc_toolButton->setChecked(false);
        ui->Vertical_doc_toolButton->setChecked(false);
    }else {
        ui->Horizontal_doc_toolButton->setEnabled(true);
        ui->Vertical_doc_toolButton->setEnabled(true);
        ui->Horizontal_doc_toolButton->setChecked(true);
        ui->Vertical_doc_toolButton->setChecked(false);
    }
}
void MainWindow::ControlSkipBlankPagesAvailable(SDIInt type)
{
    if(ui->Skip_blank_pages_comb->isEnabled()){
        ui->Skip_blank_pages_label->setEnabled(true);
        ui->Skip_blank_pages_colon_label->setEnabled(true);
    }else {
        ui->Skip_blank_pages_label->setEnabled(false);
        ui->Skip_blank_pages_colon_label->setEnabled(false);
    }
    if(ui->Skip_blank_pages_comb->isHidden()){
        ui->Skip_blank_pages_label->hide();
        ui->Skip_blank_pages_colon_label->hide();
    }else {
        ui->Skip_blank_pages_label->show();
        ui->Skip_blank_pages_colon_label->show();
    }
}
void MainWindow::ControlBackgroundColorAvailable()
{
    if(ui->Document_size_comb->current_item == Auto_Detect_long || ui->Document_size_comb->current_item == Auto_Detect){
        ui->Background_color_comb->setCurrentIndex(ui->Background_color_comb->findData(BackgroundColorBlack));
        sv->device_data.BackgroundColor.select = ui->Background_color_comb->current_item = BackgroundColorBlack;
        ui->Background_color_comb->setEnabled(false);
        goto control;
    }else {
        ui->Background_color_comb->setEnabled(true);
    }
    if(ui->Deskew_checkbox->isChecked()){
        ui->Background_color_comb->setCurrentIndex(ui->Background_color_comb->findData(BackgroundColorBlack));
        sv->device_data.BackgroundColor.select = ui->Background_color_comb->current_item = BackgroundColorBlack;
        ui->Background_color_comb->setEnabled(false);
        goto control;
    }else {
        ui->Background_color_comb->setEnabled(true);
    }
    if(sv->device_data.FunctionalUnit.select == Flatbed){
        ui->Background_color_comb->setCurrentIndex(ui->Background_color_comb->findData(BackgroundColorWhite));
        sv->device_data.BackgroundColor.select = ui->Background_color_comb->current_item = BackgroundColorWhite;
        ui->Background_color_comb->setEnabled(false);
        goto control;
    }else {
        ui->Background_color_comb->setEnabled(true);
    }
control:
    if(ui->Background_color_comb->isEnabled()){
        ui->Background_color_label->setEnabled(true);
        ui->Background_color_colon_label->setEnabled(true);
    }else {
        ui->Background_color_label->setEnabled(false);
        ui->Background_color_colon_label->setEnabled(false);
    }
    if(ui->Background_color_comb->isHidden()){
        ui->Background_color_label->hide();
        ui->Background_color_colon_label->hide();
    }else {
        ui->Background_color_label->show();
        ui->Background_color_colon_label->show();
    }
}
void MainWindow::ControlAutomaticFeedingModeAvailable()
{
    if(sv->device_data.AutomaticFeedingMode.select){
       ui->Ad_pages_checkbox->setChecked(false);
       ui->Ad_pages_checkbox->setEnabled(false);
       sv->device_data.Add_Pages = false;
    }else {
        ui->Ad_pages_checkbox->setEnabled(true);
    }
    if(ui->Automatic_feeding_mode_comb->isEnabled()){
        ui->Automatic_feeding_mode_label->setEnabled(true);
        ui->Automatic_feeding_mode_colon_label->setEnabled(true);
    }else {
        ui->Automatic_feeding_mode_label->setEnabled(false);
        ui->Automatic_feeding_mode_colon_label->setEnabled(false);
    }
    if(ui->Automatic_feeding_mode_comb->isHidden()){
        ui->Automatic_feeding_mode_label->hide();
        ui->Automatic_feeding_mode_colon_label->hide();
    }else {
        ui->Automatic_feeding_mode_label->show();
        ui->Automatic_feeding_mode_colon_label->show();
    }
}
void MainWindow::ControlDeskewAvailable()
{
    if(ui->Document_size_comb->current_item == Auto_Detect_long || ui->Document_size_comb->current_item == Auto_Detect){
        ui->Deskew_checkbox->setChecked(true);
        sv->device_data.PaperDeskew.select = true;
        ui->Deskew_checkbox->setEnabled(false);
    }else {
        ui->Deskew_checkbox->setEnabled(true);
    }
}
bool MainWindow::CheckBeforeScan()
{
    if(sv->GetScannerPositon()){
        sv->m_LastError = kSDIErrorDeviceFormUnstable;
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SCAN_109, TranslationString::GetString().translation_str.VERSION_003, Critical);
        return false;
    }
    if(sv->GetUnscannebleScanParameterStatus()){
        sv->m_LastError = kSDIErrorUnscannableDeviceConfig1;
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SCAN_107, TranslationString::GetString().translation_str.VERSION_003, Critical);
        return false;
    }
    if(sv->CheckDeviceErrorStatus() == kSDIErrorPaperJam){
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SCAN_103, TranslationString::GetString().translation_str.VERSION_003, Critical);
        return false;
    }
    if(sv->device_data.FunctionalUnit_Auto){
        if(sv->GetPaperLoadedStatus()){
            sv->device_data.FunctionalUnit.select = kSDIFunctionalUnitDocumentFeeder;
        }else {
            sv->device_data.FunctionalUnit.select = kSDIFunctionalUnitFlatbed;
        }
    }
    if(!sv->device_data.AutomaticFeedingMode.select){
        if(sv->device_data.FunctionalUnit.select == kSDIFunctionalUnitDocumentFeeder){
            if(!sv->GetPaperLoadedStatus()){
                sv->m_LastError = kSDIErrorPaperEmpty;
                m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SCAN_101, TranslationString::GetString().translation_str.VERSION_003, Warning);
                return false;
            }
        }
    }
    if(!this->SelectionRestrictions(ui->Document_size_comb->findData(ui->Document_size_comb->current_item))){
        return false;
    }
    if(sv->device_data.ScanArea.AutoSize.select == kSDIAutoSizeNone && sv->device_data.ScanArea.ScanAreaWidth.select > 21000){
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_001, TranslationString::GetString().translation_str.VERSION_003, Warning);
        return false;
    }
    if(sv->device_data.ScanArea.AutoSize.select == kSDIAutoSizeLongPaper && sv->device_data.Resolution.select > 600){
        m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_011, TranslationString::GetString().translation_str.VERSION_003, Warning);
        return false;
    }
    if(sv->device_data.ScanArea.AutoSize.select == kSDIAutoSizeNone && sv->device_data.FunctionalUnit.select == kSDIFunctionalUnitDocumentFeeder){
        if(ui->Document_size_comb->calculate_pixel_size(sv->device_data.ScanArea.ScanAreaWidth.select, 100) < sv->device_data.ScanArea.ADFWidthMin.select ||
                (ui->Document_size_comb->calculate_pixel_size(sv->device_data.ScanArea.ScanAreaHeight.select, 100) < sv->device_data.ScanArea.ADFHeightMin.select && !sv->device_data.ScanArea.PaperEndDetection.select)){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_PARA_003, TranslationString::GetString().translation_str.VERSION_003, Critical);
            return false;
        }
    }
    if(!sv->device_data.DNShow_LongPaperWarning_dialog){
        if(ui->Document_size_comb->current_item == USER_DEFINE){
            if(this->CheckLongPaperMode()){
                sv->device_data.DNShow_LongPaperWarning_dialog = m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UI_005, TranslationString::GetString().translation_str.VERSION_003, Information, true);
            }
        }
        if(ui->Document_size_comb->current_item == Auto_Detect_long){
            sv->device_data.DNShow_LongPaperWarning_dialog = m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UI_005, TranslationString::GetString().translation_str.VERSION_003, Information, true);
        }
    }
    return true;
}
void MainWindow::on_Scan_Button_pressed()
{
    ui->Scan_Button->setEnabled(false);
    ui->Scan_Button->setEnabled(true);
    this->ValidateOverRollerLifeLimit();
    this->isManualResolution();
    if(this->CheckBeforeScan()){
        ui->Prefix_lineEdit->get_prefix(&sv->device_data.Prefix);
        ui->Scan_Button->Button_pressed(sv, ui->Ad_pages_checkbox->get_current_item());
    }
    if(sv->scanning_status == ERROR){
        this->close();
    }
    sv->Get_Counter_Value();
}
void MainWindow::onSave_Button_pressed()
{
    QObject* obj = sender();
    if(obj == ui->Save_pushButton){
        m_last_used_settings_controller->CreateSettingsDict(sv->setting_file_path, sv->device_data);
    }
}
void MainWindow::onConfiguration_Button_pressed()
{
    sv->Get_Counter_Value();
    configuration_dialog->Initialize();
    configuration_dialog->UpdateCounterDisplay(sv->device_data.ConfigureCounter);
    configuration_dialog->DisplayItem(sv);
    if(configuration_dialog->isHidden()){
        configuration_dialog->exec();
    }else {
        configuration_dialog->activateWindow();
    }
    ui->Document_size_comb->item_event(-1, &sv->device_data.ScanArea);
    CONFIGURATION config_val;
    Configuration& config = Configuration::GetConfiguration();
    config_val = config.config_para;
    sv->device_data.ConfigureCounter.RollerKitCounter.select = config_val.roller_kit_counter;
    sv->bAllReset = configuration_dialog->Get_ResetFlag();
    if(sv->bAllReset){
        this->close();
    }
    if(sv->device_data.ConfigureCounter.RollerKitCounter.select == 0
    || sv->device_data.ConfigureCounter.RollerCounter.select == 0
    || sv->device_data.ConfigureCounter.RetardRollerCounter.select == 0){
        sv->Set_RollerCouunter_Value();
    }
    this->updateUI();
}
#include "savedestfolder.h"
void MainWindow::onFolder_Button_pressed()
{
    QObject* obj = sender();
    if(obj == ui->Folder_toolButton){
        const bool result = ES_CMN_FUNCS::PATH::ES_IsExistFile(sv->device_data.SelectPath);
        if (result) {
            QUrl url = QUrl::fromLocalFile( sv->device_data.SelectPath.c_str() );
            QDesktopServices::openUrl(url);
        }else {
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.E_SAVE_004, TranslationString::GetString().translation_str.VERSION_003, Warning);
        }
    }
}
void MainWindow::onAbout_Epson_Scan2_pressed()
{
    AboutEpsonScan2Dialog* about_dialog = new AboutEpsonScan2Dialog;
    if(about_dialog == NULL){
        return;
    }
    about_dialog->Initialize();
    if(about_dialog->isHidden()){
        about_dialog->exec();
    }else {
        about_dialog->activateWindow();
    }
}
void MainWindow::onHelp_Button_pressed()
{
    QString link;
    if(QLocale::system().name() == "ja_JP"){
        link = "http://download.ebz.epson.net/man_j/linux/epsonscan2_j.html";
    }else {
        link = "http://download.ebz.epson.net/man/linux/epsonscan2_e.html";
    }
    QDesktopServices::openUrl(QUrl(link));
}
void MainWindow::ValidateOverRollerLifeLimit()
{
    if(sv->device_data.ConfigureCounter.RollerKitCounter.capability.supportLevel != kSDISupportLevelNone){
        if(configuration_dialog->CheckOverRollerKitLifeLimit(sv->device_data.ConfigureCounter, &sv->device_data.OverDurationAlert)){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UNIT_001, TranslationString::GetString().translation_str.VERSION_003, Information);
            sv->device_data.NearDurationAlert = true;
        }else if(configuration_dialog->CheckNearRollerKitLifeLimit(sv->device_data.ConfigureCounter, &sv->device_data.NearDurationAlert)){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UNIT_009, TranslationString::GetString().translation_str.VERSION_003, Information);
        }
    }else {
        if(configuration_dialog->CheckOverRollerLifeLimit(sv->device_data.ConfigureCounter, &sv->device_data.OverDurationAlert)){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UNIT_001, TranslationString::GetString().translation_str.VERSION_003, Information);
            sv->device_data.NearDurationAlert = true;
        }else if(configuration_dialog->CheckNearRollerLifeLimit(sv->device_data.ConfigureCounter, &sv->device_data.NearDurationAlert)){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UNIT_009, TranslationString::GetString().translation_str.VERSION_003, Information);
        }
        if(configuration_dialog->CheckOverRetardRollerLifeLimit(sv->device_data.ConfigureCounter, &sv->device_data.OverDurationAlert)){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UNIT_001, TranslationString::GetString().translation_str.VERSION_003, Information);
            sv->device_data.NearDurationAlert = true;
        }else if(configuration_dialog->CheckNearRetardRollerLifeLimit(sv->device_data.ConfigureCounter, &sv->device_data.NearDurationAlert)){
            m_messageBox->CreateMessageBox(TranslationString::GetString().translation_str.A_UNIT_009, TranslationString::GetString().translation_str.VERSION_003, Information);
        }
    }
}
